var Extension = Extension || {};

Extension.isArmLineCollidingMesh = function(armMesh, planeNormal, center) {
    var s = armMesh.boundingBox.getSize();
    var diff = new THREE.Vector3(s.x*Math.abs(planeNormal.x)/2, s.y*Math.abs(planeNormal.y)/2, s.z*Math.abs(planeNormal.z)/2);
    var start  = center.clone().sub(diff);
    //var end  = center.clone().add(diff);
    var ray = new THREE.Ray(start, diff.clone().normalize());
    var faces = armMesh.geometry.faces
    for (var k = 0; k < faces.length; k++) {
        var face = faces[k];
        var v1 = armMesh.geometry.vertices[face.a];
        var v2 = armMesh.geometry.vertices[face.b];
        var v3 = armMesh.geometry.vertices[face.c];
        var intPoint = ray.intersectTriangle(v1, v2, v3, false);
        if(intPoint !== null) return true;
    }
    return false;
}

Extension.preparePossiblePoints = function(bb, center) {
    var walls = [];
    walls.push(new THREE.Vector3(bb.max.x + 0.001, center.y, center.z));
    walls.push(new THREE.Vector3(bb.min.x - 0.001, center.y, center.z));
    walls.push(new THREE.Vector3(center.x, bb.max.y + 0.001, center.z));
    walls.push(new THREE.Vector3(center.x, bb.min.y - 0.001, center.z));
    walls.push(new THREE.Vector3(center.x, center.y, bb.max.z + 0.001));
    walls.push(new THREE.Vector3(center.x, center.y, bb.min.z - 0.001));
    return walls;
}

Extension.moveCenterOutsideMesh = function(center, armMesh) {
    var bb = armMesh.boundingBox;
    var points = Extension.preparePossiblePoints(bb, center);
    var minDistance = 9999999; var minPoint = null;
    for (var j = 0; j < points.length; j++) {
        var point = points[j];
        var dist = center.distanceTo(point);
        if(dist < minDistance) {
            minDistance = dist;
            minPoint = point;
        }
    }
    return minPoint;
}

Extension.calculateArmLine = function(mesh, armMesh, plane, ignoreCollision) {
    var center = armMesh.boundingBox.getCenter();
    var isColliding = Extension.isArmLineCollidingMesh(armMesh, plane.normal, center);
    if(isColliding && ignoreCollision !== true) {
        var oldCenter = center;
        center = Extension.moveCenterOutsideMesh(center, mesh);
        var centerDiff = center.clone().sub(oldCenter);
        armMesh.armStart = armMesh.armStart.clone().add(centerDiff);
    }
    var n = plane.normal.clone().multiplyScalar(3000);
    var start = new THREE.Vector3(center.x - Math.abs(n.x), center.y - Math.abs(n.y), center.z - Math.abs(n.z));
    var end = new THREE.Vector3(center.x + Math.abs(n.x), center.y + Math.abs(n.y), center.z + Math.abs(n.z));
    return new THREE.Line3(start, end);
}

Extension.checkArmLineConnections = function(arm, arms) {
    var count = 1;
    for (var j = 0; j < arms.length; j++) {
        var otherArm = arms[j];
        if(otherArm != arm && otherArm.isValid) {//change this condition
            var dist = Extension.calculateDistanceBetweenLines(arm.line.start, arm.line.end, otherArm.line.start, otherArm.line.end);
            if(dist < 0.003) {//must be checked a little bit different
                count +=1;//if line intersects with other arm candidate then it is more likely to be an arm
            } else {
                count -=1; //if not it is possibly not an arm
            }
        }
    }
    return count > 0;
}

Extension.clampArmDir = function(dir) {
    function clampDim(val) {
        if(Math.abs(val) < 0.05) return 0;
        if(val > 0.95) return 1;
        if(val < -0.95) return -1;
        return val;
    }
    return new THREE.Vector3(clampDim(dir.x),clampDim(dir.y),clampDim(dir.z));
}
Extension.calculateArmDir = function(armMesh, intersectPoint) {
    return Extension.clampArmDir(armMesh.armStart.clone().sub(intersectPoint));
}

Extension.processArms = function(arms) {
    var validArms = [];
    for(var i = 0 ; i < arms.length ; i++) {
        var arm = arms[i];
        if(arm.isValid && Extension.checkArmLineConnections(arm, arms)) {
            validArms.push(arm);
        }
    }

    if(validArms.length > 1) {//if more then 2 arms then center is in intersection between them
        var intersect = Extension.intersectionPoint(
            validArms[0].line.start, validArms[0].line.end, validArms[1].line.start, validArms[1].line.end);
        Extension.armsIntersection = intersect.point;
        Extension.pointMesh = Extension.createPointMesh(intersect.point);
        scene.add(Extension.pointMesh);
        for(var i = 0 ; i < validArms.length ; i++) {
            var arm = validArms[i];
            Extension.newArmLength = Extension.calculateArmDir(arm.mesh, intersect.point).length();
            arm.lineGroupMesh = Extension.createArmLineMesh(arm.mesh, intersect.point, i+1);
            scene.add(arm.lineGroupMesh);
            Extension.arms.push(Extension.createArmDefiniton(arm, i));
        }
    }
}

Extension.createArm = function(mesh, position, index, plane, color) {
    var armTriangles = Extension.processPlaneIntersection(mesh, position, index, plane, color);
    var armMesh = Extension.createTrianglesMesh(armTriangles, color, plane.normal);
    
    var armValidator = new Extension.ConnectorArmValidator(armMesh, plane);
    return { mesh : armMesh, isValid : armValidator.isValidArm(), line : Extension.calculateArmLine(mesh, armMesh, plane) };
}


Extension.recogniseArms = function(mesh, position, index) {
    var bbox = new THREE.Box3().setFromObject(mesh);
    Extension.modelConfigBoundingBoxMin = bbox.min;
    Extension.modelConfigBoundingBoxMax = bbox.max;
    var arms = [];

    var planeX = new THREE.Plane(new THREE.Vector3(-1,0,0), bbox.min.x);
    arms.push(Extension.createArm(mesh, position, index, planeX, new THREE.Vector3(1,0,0)));
    var planeZ = new THREE.Plane(new THREE.Vector3(0,0,-1), bbox.min.z);
    arms.push(Extension.createArm(mesh, position, index, planeZ, new THREE.Vector3(0,0,1)));
    var planeY = new THREE.Plane(new THREE.Vector3(0,-1,0), bbox.min.y);
    arms.push(Extension.createArm(mesh, position, index, planeY, new THREE.Vector3(0,1,0)));
    var planeX2 = new THREE.Plane(new THREE.Vector3(1,0,0), -bbox.max.x);
    arms.push(Extension.createArm(mesh, position, index, planeX2, new THREE.Vector3(1,1,0)));
    var planeZ2 = new THREE.Plane(new THREE.Vector3(0,0,1), -bbox.max.z);
    arms.push(Extension.createArm(mesh, position, index, planeZ2, new THREE.Vector3(0,1,1)));
    var planeY2 = new THREE.Plane(new THREE.Vector3(0,1,0), -bbox.max.y);
    arms.push(Extension.createArm(mesh, position, index, planeY2, new THREE.Vector3(1,0,1)));

    Extension.processArms(arms);
}