(function () {
    'use strict';
    describe('MODEL.types', function () {
        describe('Vector', function () {
            it(' is Immutable', function () {
                var vector = new MODEL.types.Vector(1, 2, 3);
                expect(function () { vector.x = 5; }).toThrow();
            });
        });
    });

    describe('ConstructionViewModel', function () {
        describe('addNode', function () {
            it('allows to add a node', function () {
                var model = new MODEL.ConstructionViewModel();
                model.addNode(new MODEL.Node(new MODEL.types.Vector(0, 0, 0)));
                expect(model.nodes().length).toBe(1);
            });
            it('will not let to insert the same node twice', function () {
                var model = new MODEL.ConstructionViewModel();
                var node = model.addNode(new MODEL.Node(new MODEL.types.Vector(0, 0, 0)));
                expect(function () { model.addNode(node); }).toThrow();
            });
            it('will not let to insert null', function () {
                var model = new MODEL.ConstructionViewModel();
                expect(function () { model.addNode(null); }).toThrow();
            });
        });

        describe('removeNode', function () {
            it('allows to remove a node', function () {
                var model = new MODEL.ConstructionViewModel();
                var node  = new MODEL.Node(new MODEL.types.Vector(0, 0, 0))
                model.addNode(node);
                model.removeNode(node);
                expect(model.nodes().length).toBe(0);
            });
            it('will throw exception if you try to remove node which he dont own', function () {
                var model = new MODEL.ConstructionViewModel();
                var node = new MODEL.Node(new MODEL.types.Vector(0, 0, 0))
                expect(function () { model.removeNode(node); }).toThrow();
            });
        });

        describe('addBar', function () {
            it('allows to add a bar', function () {
                var model = new MODEL.ConstructionViewModel();
                var node1 = model.addNode(new MODEL.Node(new MODEL.types.Vector(0, 0, 0)));
                var node2 = model.addNode( new MODEL.Node(new MODEL.types.Vector(1, 0, 0)));
                model.addBar(new MODEL.Bar(node1, node2));
                expect(model.bars().length).toBe(1);
            });
            it('will not let to insert the same bar twice', function () {
                var model = new MODEL.ConstructionViewModel();
                var node1 = model.addNode(new MODEL.Node(new MODEL.types.Vector(0, 0, 0)));
                var node2 = model.addNode(new MODEL.Node(new MODEL.types.Vector(1, 0, 0)));
                var bar = new MODEL.Bar(node1, node2);
                model.addBar(bar);
                expect(function () { model.addBar(bar); }).toThrow();
            });
            it('will not let to insert a bar with node which in not in the model', function () {
                var model = new MODEL.ConstructionViewModel();
                var node1 = new MODEL.Node(new MODEL.types.Vector(0, 0, 0));
                var node2 = new MODEL.Node(new MODEL.types.Vector(1, 0, 0));
                var bar = new MODEL.Bar(node1, node2);
                expect(function () { model.addBar(bar); }).toThrow();
            });
            it('will not let to insert null', function () {
                var model = new MODEL.ConstructionViewModel();
                expect(function () { model.addBar(null); }).toThrow();
            });
        });

        describe('removeBar', function () {
            it('allows to remove a bar', function () {
                var model = new MODEL.ConstructionViewModel();
                var node1 = model.addNode(new MODEL.Node(new MODEL.types.Vector(0, 0, 0)));
                var node2 = model.addNode(new MODEL.Node(new MODEL.types.Vector(1, 0, 0)));
                var bar = new MODEL.Bar(node1, node2);
                model.addBar(bar);
                model.removeBar(bar);
                expect(model.bars().length).toBe(0);
            });
            it('cleans references in connected points', function () {
                var model = new MODEL.ConstructionViewModel();
                var node1 = model.addNode(new MODEL.Node(new MODEL.types.Vector(0, 0, 0)));
                var node2 = model.addNode(new MODEL.Node(new MODEL.types.Vector(1, 0, 0)));
                var bar = new MODEL.Bar(node1, node2);
                model.addBar(bar);
                model.removeBar(bar);
                expect(node1.createsBars().length).toBe(0);
            });
            it("will throw exception if you try to remove bar which he don't own", function () {
                var model = new MODEL.ConstructionViewModel();
                var node1 = model.addNode(new MODEL.Node(new MODEL.types.Vector(0, 0, 0)));
                var node2 = model.addNode(new MODEL.Node(new MODEL.types.Vector(1, 0, 0)));
                var bar = new MODEL.Bar(node1, node2);
                expect(function () { model.removeBar(bar); }).toThrow();
            });
        });

        describe('addLoad', function () {
            it('allows to add a load', function () {
                var model = new MODEL.ConstructionViewModel();
                model.addLoad(new MODEL.load.node.NodeForce());
                expect(model.loads().length).toBe(1);
            });
            it('will not let to insert the same load twice', function () {
                var model = new MODEL.ConstructionViewModel();
                var load = model.addLoad(new MODEL.load.node.NodeForce());
                expect(function () { model.addLoad(load); }).toThrow();
            });
            it('will not let to insert null', function () {
                var model = new MODEL.ConstructionViewModel();
                expect(function () { model.addLoad(null); }).toThrow();
            });
        });

        describe('removeLoad', function () {
            it('allows to remove a load', function () {
                var model = new MODEL.ConstructionViewModel();
                var load = model.addLoad(new MODEL.load.node.NodeForce());
                model.removeLoad(load);
                expect(model.loads().length).toBe(0);
            });
            it('will throw exception if you try to remove load which he dont own', function () {
                var model = new MODEL.ConstructionViewModel();
                var load = new MODEL.load.node.NodeForce();
                expect(function () { model.removeLoad(load); }).toThrow();
            });
        });

        describe('addWorkplane', function () {
            it('allows to add a workplane', function () {
                var model = new MODEL.ConstructionViewModel();
                model.addWorkplane(new MODEL.Workplane(
                    new MODEL.types.ThreeVectors(
                        new MODEL.types.Vector(1,0,0),
                        new MODEL.types.Vector(0,1,0),
                        new MODEL.types.Vector(0,0,1))));
                expect(model.workplanes().length).toBe(1);
            });
            it('will not let to insert the same workplane twice', function () {
                var model = new MODEL.ConstructionViewModel();
                var workplane = model.addWorkplane(new MODEL.Workplane(
                    new MODEL.types.ThreeVectors(
                        new MODEL.types.Vector(1, 0, 0),
                        new MODEL.types.Vector(0, 1, 0),
                        new MODEL.types.Vector(0, 0, 1))));
                expect(function () { model.addWorkplane(workplane); }).toThrow();
            });
            it('will not let to insert null', function () {
                var model = new MODEL.ConstructionViewModel();
                expect(function () { model.addWorkplane(null); }).toThrow();
            });
        });

        describe('removeWorkplane', function () {
            it('allows to remove a workplane', function () {
                var model = new MODEL.ConstructionViewModel();
                var workplane = model.addWorkplane(new MODEL.Workplane(
                    new MODEL.types.ThreeVectors(
                        new MODEL.types.Vector(1, 0, 0),
                        new MODEL.types.Vector(0, 1, 0),
                        new MODEL.types.Vector(0, 0, 1))));
                model.removeWorkplane(workplane);
                expect(model.workplanes().length).toBe(0);
            });
            it('will throw exception if you try to remove workplane which he dont own', function () {
                var model = new MODEL.ConstructionViewModel();
                var workplane = new MODEL.Workplane(
                    new MODEL.types.ThreeVectors(
                        new MODEL.types.Vector(1, 0, 0),
                        new MODEL.types.Vector(0, 1, 0),
                        new MODEL.types.Vector(0, 0, 1)));
                expect(function () { model.removeWorkplane(workplane); }).toThrow();
            });
        });

        describe('watch', function () {
            
            it('notifies if Node was added', function () {
                var model = new MODEL.ConstructionViewModel();
                var notified = false;
                model.watchModel(function () { notified = true; });
                model.addNode(new MODEL.Node(new MODEL.types.Vector(0, 0, 0)));
                expect(notified).toBe(true);
            });
            it('notifies if Node was changed', function () {
                var model = new MODEL.ConstructionViewModel();
                var count = 0;
                var node = model.addNode(new MODEL.Node(new MODEL.types.Vector(0, 0, 0)));
                model.watchModel(function () { count++; }, false);
                node.position(new MODEL.types.Vector(1, 0, 0));
                expect(count).toBe(1);
            });
            it('notifies if added Node was changed', function () {
                var model = new MODEL.ConstructionViewModel();
                var count = 0;
                model.watchModel(function () { count++; });
                var node = model.addNode(new MODEL.Node(new MODEL.types.Vector(0, 0, 0)));
                expect(count).toBe(1);
                node.position(new MODEL.types.Vector(1, 0, 0));
                expect(count).toBe(2);
            });
            it('stops notify if Node was removed', function () {
                var model = new MODEL.ConstructionViewModel();
                var count = 0;
                var node = model.addNode(new MODEL.Node(new MODEL.types.Vector(0, 0, 0)));
                model.watchModel(function () { count++; });
                model.removeNode(node);
                expect(count).toBe(1);
                node.position(new MODEL.types.Vector(1, 0, 0));
                expect(count).toBe(1); 
            });
            it('notifies asigned Load change only once', function () {
                var model = new MODEL.ConstructionViewModel();
                var count = 0;
                var node = model.addNode(new MODEL.Node(new MODEL.types.Vector(0, 0, 0)));
                var load = model.addLoad(new MODEL.load.node.NodeForce());
                load.assignTo(node);
                model.watchModel(function () { count++; });
                node.loads()[0].forces(new MODEL.types.Vector(1, 0, 0));
                expect(count).toBe(1);
            });
        });
    });
})();
