﻿(function () {
    'use strict';
    var helpers = {
        addNode(nodes, node, id) {
            var n = new MODEL.Node(new MODEL.types.Vector(node.x, node.y, node.z));
            n.id = id;
            nodes.push(n);
            return n;
        },
        validateVector(current, expected) {
            expect(current.x).toBe(expected.x);
            expect(current.y).toBe(expected.y);
            expect(current.z).toBe(expected.z);
        },
        validateMesurement(mesurement, start, lenght) {
            this.validateVector(mesurement.start(), start);
            expect(mesurement.length()).toBe(lenght);

        }
    }

    describe('MODEL.Measurements', function () {
        describe('onAddNode', function () {
            it("will not create any measurement for first node", function () {
                var nodes = ko.observableArray();
                var measurements = new MODEL.Measurements(nodes);
                helpers.addNode(nodes, { x: 1, y: 2, z: 3 }, 1);

                expect(measurements.xChain().length).toBe(0);
                expect(measurements.yChain().length).toBe(0);
                expect(measurements.zChain().length).toBe(0);
            });
            it('will will create measurements', function () {
                var nodes = ko.observableArray();
                var measurements = new MODEL.Measurements(nodes);

                helpers.addNode(nodes, { x: 1, y: 2, z: 3 }, 1);
                helpers.addNode(nodes, { x: 2, y: 4, z: 6 }, 2);

                expect(measurements.xChain().length).toBe(1);
                helpers.validateMesurement(measurements.xChain()[0], { x: 1, y: 2, z: 3 }, 1);
                expect(measurements.yChain().length).toBe(1);
                helpers.validateMesurement(measurements.yChain()[0], { x: 1, y: 2, z: 3 }, 2);
                expect(measurements.zChain().length).toBe(1);
                helpers.validateMesurement(measurements.zChain()[0], { x: 1, y: 2, z: 3 }, 3);
            });
            it("will not create any measurement if new node lies on existing border", function () {
                var nodes = ko.observableArray();
                helpers.addNode(nodes, { x: 1, y: 0, z: 0 }, 1);
                helpers.addNode(nodes, { x: 3, y: 0, z: 0 }, 2);
                helpers.addNode(nodes, { x: 2, y: 0, z: 0 }, 3);
                var measurements = new MODEL.Measurements(nodes);

                helpers.addNode(nodes, { x: 2, y: 0, z: 0 }, 2);
                expect(measurements.xChain().length).toBe(2);
                helpers.validateMesurement(measurements.xChain()[0], { x: 1, y: 0, z: 0 }, 1);
                helpers.validateMesurement(measurements.xChain()[1], { x: 2, y: 0, z: 0 }, 1);
            });
        });
        
        describe('onRemoveNode', function () {
            it("will remove mesurement if last node was removed", function () {
                var nodes = ko.observableArray();
                helpers.addNode(nodes, { x: 3, y: 0, z: 0 }, 1);
                helpers.addNode(nodes, { x: 4, y: 0, z: 0 }, 2);//<-- to be removed
                helpers.addNode(nodes, { x: 7, y: 0, z: 0 }, 3);
                var measurements = new MODEL.Measurements(nodes);

                nodes.splice(1, 1);//remove second node

                expect(measurements.xChain().length).toBe(1);
                helpers.validateMesurement(measurements.xChain()[0], { x: 3, y: 0, z: 0 }, 7-3);
            });
            it("will not remove mesurement if any nodeleft", function () {
                var nodes = ko.observableArray();
                helpers.addNode(nodes, { x: 3, y: 0, z: 0 }, 1);
                helpers.addNode(nodes, { x: 4, y: 0, z: 0 }, 2);//<-- to be removed
                helpers.addNode(nodes, { x: 4, y: 1, z: 0 }, 3);
                var measurements = new MODEL.Measurements(nodes);

                nodes.splice(1, 1);//remove second node

                expect(measurements.xChain().length).toBe(1);
                helpers.validateMesurement(measurements.xChain()[0], { x: 3, y: 0, z: 0 }, 1);
            });
        });
        describe('onMesurementLenghtChange', function () {
            it("will move nodes on this mesurement and on right of mesurement", function () {
                var nodes = ko.observableArray();
                helpers.addNode(nodes, { x: 0, y: 6, z: 0 }, 1);
                var node1 = helpers.addNode(nodes, { x: 0, y: 7, z: 0 }, 2);
                var node2 = helpers.addNode(nodes, { x: 0, y: 8, z: 0 }, 2);
                var measurements = new MODEL.Measurements(nodes);

                measurements.yChain()[0].length(2);

                expect(node1.position().y).toBe(8);
                expect(node2.position().y).toBe(9);
            });
            it("will update mesurements on right", function () {
                var nodes = ko.observableArray();
                helpers.addNode(nodes, { x: 0, y: 0, z: 0 }, 1);
                helpers.addNode(nodes, { x: 2, y: 0, z: 0 }, 2);
                helpers.addNode(nodes, { x: 4, y: 0, z: 0 }, 3);
                var measurements = new MODEL.Measurements(nodes);

                measurements.xChain()[0].length(1);

                helpers.validateMesurement(measurements.xChain()[1], { x: 1, y: 0, z: 0 }, 2);
            });
        });
        describe('onNodePositionChange', function () {
            it("will move mesureent if it is last in group", function () {
                var nodes = ko.observableArray();
                helpers.addNode(nodes, { x: 0, y: 0, z: 0 }, 1);
                var node = helpers.addNode(nodes, { x: 1, y: 0, z: 0 }, 2);
                var measurements = new MODEL.Measurements(nodes);
                node.position(new MODEL.types.Vector(2, 0, 0));
                helpers.validateMesurement(measurements.xChain()[0], { x: 0, y: 0, z: 0 }, 2);
            });
            it("will remove mesureent", function () {
                var nodes = ko.observableArray();
                helpers.addNode(nodes, { x: 0, y: 0, z: 0 }, 1);
                var node = helpers.addNode(nodes, { x: 1, y: 0, z: 0 }, 2);
                helpers.addNode(nodes, { x: 2, y: 1, z: 0 }, 3);

                var measurements = new MODEL.Measurements(nodes);
                node.position(new MODEL.types.Vector(0, 0, 0));

                expect(measurements.xChain().length).toBe(1);
            });
        });
    });
})();
